# Generated by Django 3.2.13 on 2022-05-15 23:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0003_auto_20220506_2242'),
    ]

    operations = [
        migrations.AddField(
            model_name='h2fingerprint',
            name='priority_frames',
            field=models.TextField(blank=True, null=True),
        ),
    ]
