# Generated by Django 3.2.13 on 2022-05-29 00:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djspoofer', '0006_auto_20220515_2357'),
    ]

    operations = [
        migrations.AlterField(
            model_name='h2fingerprint',
            name='header_table_size',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]
